## \file
## \ingroup tutorial_tmva
## \notebook
## TMVA example, for classification
##  with following objectives:
##  * Apply a BDT with TMVA
##
## \macro_image
## \macro_output
## \macro_code
##
## \author Lailin XU

# from ROOT import TMVA, TFile, TTree, TCut, TH1F, TCanvas, gROOT, TLegend, TChain
from ROOT import *
from subprocess import call
from os.path import isfile
from array import array
import glob

mtagList = ['m30','m100','m200','m500','m1000']
miniTuplePath = '../MiniTuple'
varList = [
    # 'pdgId',
    # 'mass', ## not branch
    # 'beta', 'pixelHit', 'sctHit', 'trtHit', 'npl',
    'beta',
    'pixeldEdx', 'mdtdEdx',
    # 'pixeldEdxHit'
]

lumi = 139.
dict = {} # fb
dict['c1030m30'] = 7.5e5
dict['c1030m100'] = 4.71e2
dict['c1030m200'] = 3.65e1
dict['c1030m500'] = 8.13e-1
dict['c1030m1000'] = 2.09e-2

dict['c1020m30'] = 7.55e5
dict['c1020m100'] = 6.36e2
dict['c1020m200'] = 5.11e1
dict['c1020m500'] = 1.17
dict['c1020m1000'] = 3.1e-2

dict['c2030m30'] = 7.62e5
dict['c2030m100'] = 8.52e2
dict['c2030m200'] = 7.03e1
dict['c2030m500'] = 1.64
dict['c2030m1000'] = 4.43e-2

dict['Zmumu'] = 1e6*1.9011*1.026
 
gROOT.SetStyle("ATLAS")

# Setup TMVA
TMVA.Tools.Instance()

inputFolderName = miniTuplePath
for mtag in mtagList:
  sigTag = 'c2030'+mtag

  # Reader. One reader for each application.
  reader = TMVA.Reader("Color:!Silent")
  reader_S = TMVA.Reader("Color:!Silent")
  reader_B = TMVA.Reader("Color:!Silent")
  
  # Inputs
  # =============
  # Load data

  c2030FileList = []
  ZmumuFileList = []
  # c2030FileList.extend(glob.glob(r''+inputFolderName+'/*2030_1000*FCP.root'))
  c2030FileList.extend(glob.glob(r''+inputFolderName+'/*'+(sigTag.split('m')[0]).split('c')[-1]+'_'+sigTag.split('m')[-1]+'*FCP.root'))
  ZmumuFileList.extend(glob.glob(r''+inputFolderName+'/*Zmumu.root'))
  c2030Chain = TChain('mini')
  ZmumuChain = TChain('mini')
  # mixedChain = TChain('mini')

  for f in c2030FileList:
    c2030Chain.Add(f)
    # mixedChain.Add(f)
  for f in ZmumuFileList:
    ZmumuChain.Add(f)
    # mixedChain.Add(f)

  # tree = mixedChain
  tree_S = c2030Chain
  tree_B = ZmumuChain

  # Set input variables. Do this for each reader
  # branches = {}
  # for branch in tree.GetListOfBranches():
  #   branchName = branch.GetName()
  #   branches[branchName] = array('f', [-999])
  #   tree.SetBranchAddress(branchName, branches[branchName])
  #   if branchName in varList:
  #     reader.AddVariable(branchName, branches[branchName])

  branches_S = {}
  for branch in tree_S.GetListOfBranches():
    branchName = branch.GetName()
    branches_S[branchName] = array('f', [-999])
    tree_S.SetBranchAddress(branchName, branches_S[branchName])
    if branchName in varList:
      reader_S.AddVariable(branchName, branches_S[branchName])

  branches_B = {}
  for branch in tree_B.GetListOfBranches():
    branchName = branch.GetName()
    branches_B[branchName] = array('f', [-999])
    tree_B.SetBranchAddress(branchName, branches_B[branchName])
    if branchName in varList:
      reader_B.AddVariable(branchName, branches_B[branchName])

  # Book method(s)
  # =============
  # MLP_ANN
  methodName1 = "MLP_ANN"
  weightfile = 'DAOD/weights/TMVA_{0}.weights.xml'.format(methodName1)
  # reader.BookMVA( methodName1, weightfile )
  reader_S.BookMVA( methodName1, weightfile )
  reader_B.BookMVA( methodName1, weightfile )

  # BDTG
  methodName2 = "BDTG"
  weightfile = 'DAOD/weights/TMVA_{0}.weights.xml'.format(methodName2)
  # reader.BookMVA( methodName2, weightfile )
  reader_S.BookMVA( methodName2, weightfile )
  reader_B.BookMVA( methodName2, weightfile )

  # Loop events for evaluation
  # ================

  # Book histograms
  nbins, xmin, xmax=25, -1, 1
  nbins, xmin1, xmax1=25, 0, 1
  # Signal
  tag = "S"
  hname="MLP_ANN{0}".format(tag)
  h1 = TH1F(hname, hname, nbins, xmin1, xmax1)
  h1.Sumw2()
  hname="BDTG_{0}".format(tag)
  h2 = TH1F(hname, hname, nbins, xmin, xmax)
  h2.Sumw2()

  nevents = tree_S.GetEntries()
  for i in range(nevents):
    tree_S.GetEntry(i)
    MLP_ANN = reader_S.EvaluateMVA(methodName1)
    BDTG = reader_S.EvaluateMVA(methodName2)
    h1.Fill(MLP_ANN, lumi*dict[sigTag]*(tree_S.overallWeight/tree_S.sumOfWeight))
    h2.Fill(BDTG, lumi*dict[sigTag]*(tree_S.overallWeight/tree_S.sumOfWeight))

  # Background
  tag = "B"
  bkgTag = "Zmumu"
  hname="MLP_ANN{0}".format(tag)
  h3 = TH1F(hname, hname, nbins, xmin1, xmax1)
  h3.Sumw2()
  hname="BDTG_{0}".format(tag)
  h4 = TH1F(hname, hname, nbins, xmin, xmax)
  h4.Sumw2()

  nevents = tree_B.GetEntries()
  for i in range(nevents):
    tree_B.GetEntry(i)
    MLP_ANN = reader_B.EvaluateMVA(methodName1)
    BDTG = reader_B.EvaluateMVA(methodName2)
    h3.Fill(MLP_ANN, lumi*dict[bkgTag]*(tree_B.overallWeight/tree_B.sumOfWeight))
    h4.Fill(BDTG, lumi*dict[bkgTag]*(tree_B.overallWeight/tree_B.sumOfWeight))

  # # Mixed sample
  # tag = "Mix"
  # hname="MLP_ANN{0}".format(tag)
  # h5 = TH1F(hname, hname, nbins, xmin1, xmax1)
  # h5.Sumw2()
  # hname="BDTG_{0}".format(tag)
  # h6 = TH1F(hname, hname, nbins, xmin, xmax)
  # h6.Sumw2()

  # nevents = tree.GetEntries()
  # for i in range(nevents):
  #   tree.GetEntry(i)
  #   MLP_ANN = reader.EvaluateMVA(methodName1)
  #   BDTG = reader.EvaluateMVA(methodName2)
  #   h5.Fill(MLP_ANN)
  #   h6.Fill(BDTG)

  # Helper function to normalize hists
  def norm_hists(h):
    h_new = h.Clone()
    hname = h.GetName() + "_normalized"
    h_new.SetName(hname)
    h_new.SetTitle(hname)
    ntot = h.Integral()
    if ntot!=0:
      h_new.Scale(1./ntot)
    return h_new

  # Plotting
  myc = TCanvas("c", "c", 800, 600)
  myc.SetFillColor(0)
  # myc.SetLogy()
  myc.cd()

  # # Compare the performance for MLP_ANN
  outFileName = '../results/MLP_App_'+sigTag+'.root'
  outFile = TFile.Open(outFileName, 'RECREATE')
  nh1 = norm_hists(h1)
  nh1.GetXaxis().SetTitle("MLP_ANN")
  nh1.GetYaxis().SetTitle("Norm.Events")
  nh1.Draw("hist")
  nh3 = norm_hists(h3)
  nh3.SetLineColor(2)
  nh3.SetMarkerColor(2)
  nh3.Draw("same hist")
  # nh5 = norm_hists(h5)
  # nh5.SetLineColor(4)
  # nh5.SetMarkerColor(4)
  # nh5.Draw("same")

  ymin = 0
  ymax = max(nh1.GetMaximum(), nh3.GetMaximum()) #, nh5.GetMaximum()
  nh1.GetYaxis().SetRangeUser(ymin, ymax*1.5)

  # Draw legends
  lIy = 0.92
  lg = TLegend(0.60, lIy-0.25, 0.85, lIy)
  lg.SetBorderSize(0)
  lg.SetFillStyle(0)
  lg.SetTextFont(42)
  lg.SetTextSize(0.04)
  lg.AddEntry(nh1, "Signal", "l")
  lg.AddEntry(nh3, "Background", "l")
  # lg.AddEntry(nh5, "Mixed", "l")
  lg.Draw()

  myc.Draw()
  myc.SaveAs("../results/MLP_App_"+sigTag+".png")

  h1.Write()
  h3.Write()
  # h5.Write()
  outFile.Close()

  # Compare the performance for BDTG
  outFileName = '../results/BDTG_App_'+sigTag+'.root'
  outFile = TFile.Open(outFileName, 'RECREATE')

  nh1 = norm_hists(h2)
  nh1.GetXaxis().SetTitle("BDTG")
  nh1.GetYaxis().SetTitle("Norm.Events")
  nh1.Draw("hist")
  nh3 = norm_hists(h4)
  nh3.SetLineColor(2)
  nh3.SetMarkerColor(2)
  nh3.Draw("same hist")
  # nh5 = norm_hists(h6)
  # nh5.SetLineColor(4)
  # nh5.SetMarkerColor(4)
  # nh5.Draw("same")

  ymin = 0
  ymax = max(nh1.GetMaximum(), nh3.GetMaximum()) #, nh5.GetMaximum()
  nh1.GetYaxis().SetRangeUser(ymin, ymax*1.2)

  # Draw legends
  lIy = 0.92
  lg = TLegend(0.60, lIy-0.25, 0.85, lIy)
  lg.SetBorderSize(0)
  lg.SetFillStyle(0)
  lg.SetTextFont(42)
  lg.SetTextSize(0.04)
  lg.AddEntry(nh1, "Signal", "l")
  lg.AddEntry(nh3, "Background", "l")
  # lg.AddEntry(nh5, "Mixed", "l")
  lg.Draw()

  myc.Draw()
  myc.SaveAs("../results/BDTG_App_"+sigTag+".png")

  # print("+++++++++",nh3.GetEntries())

  h2.Write()
  h4.Write()
  # h6.Write()
  outFile.Close()
  del reader, reader_S, reader_B, h1, h2, h3, h4, nh1, nh3
