## \file
## \ingroup tutorial_tmva
## \notebook
## TMVA example, for classification
##  with following objectives:
##  * Train a BDT with TMVA
##
## \macro_image
## \macro_output
## \macro_code
##
## \author Lailin XU
## Modified from [ClassificationKeras.py](https://root.cern/doc/master/ClassificationKeras_8py.html) and [TMVAClassification.C](https://root.cern/doc/master/TMVAClassification_8C.html)

from ROOT import TMVA, TFile, TTree, TCut, TChain, gROOT
# from ROOT import *
from subprocess import call
from os.path import isfile
import glob

miniTuplePath = '../MiniTuple'
parList = ['c2030', 'Zmumu']
lumi = 139.

# dict = {} # fb
# dict['c1030m30'] = 7.5e5
# dict['c1030m100'] = 4.71e2
# dict['c1030m200'] = 3.65e1
# dict['c1030m500'] = 8.13e-1
# dict['c1030m1000'] = 2.09e-2

# dict['c1020m30'] = 7.55e5
# dict['c1020m100'] = 6.36e2
# dict['c1020m200'] = 5.11e1
# dict['c1020m500'] = 1.17
# dict['c1020m1000'] = 3.1e-2

# dict['c2030m30'] = 7.62e5
# dict['c2030m100'] = 8.52e2
# dict['c2030m200'] = 7.03e1
# dict['c2030m500'] = 1.64
# dict['c2030m1000'] = 4.43e-2

# dict['Zmumu'] = 1e6*1.9011*1.026
 
# Setup TMVA
# =======================
TMVA.Tools.Instance()
# (TMVA.gConfig().GetVariablePlotting()).fMaxNumOfAllowedVariablesForScatterPlots = 5 
 
outfileName = 'c2030.root'
output = TFile.Open(outfileName, 'RECREATE')
# Create the factory object. Later you can choose the methods whose performance you'd like to investigate. The factory is
#    the only TMVA object you have to interact with
#   
#    The first argument is the base of the name of all the weightfiles in the directory weight/
#    The second argument is the output file for the training results
factory = TMVA.Factory("TMVA", output,
  "!V:!Silent:Color:DrawProgressBar:Transformations=I;D;P;G,D:AnalysisType=Classification")
 
# Load data
# =======================
inputFolderName = miniTuplePath
c2030FileList = []
ZmumuFileList = []

c2030FileList.extend(glob.glob(r''+inputFolderName+'/*2030*FCP.root'))
ZmumuFileList.extend(glob.glob(r''+inputFolderName+'/*Zmumu.root'))

c2030Chain = TChain('mini')
ZmumuChain = TChain('mini')

for f in c2030FileList:
  c2030Chain.Add(f)
for f in ZmumuFileList:
  ZmumuChain.Add(f)

# for p in parList:
#   exec("p+'Tree' = p+'Chain'.GetTree()")

varList = [
    # 'pdgId',
    # 'mass', ## not branch
    # 'beta', 'pixelHit', 'sctHit', 'trtHit', 'npl',
    'beta',
    'pixeldEdx', 'mdtdEdx',
    # 'pixeldEdxHit'
]
dataloader = TMVA.DataLoader('DAOD')
for branch in c2030Chain.GetListOfBranches():
  name = branch.GetName()
  if name in varList:
    dataloader.AddVariable(name)

# Add signal and background trees
for p in parList:
  if 'c' in p:
    # exec("sigTree = p+'Tree'")
    dataloader.AddSignalTree(c2030Chain, 1.0) #lumi
  else:
    # exec("bkgTree = p+'Tree'")
    dataloader.AddBackgroundTree(ZmumuChain, 1.0) #lumi

# Set individual event weights (the variables must exist in the original TTree)
# dataloader.SetSignalWeightExpression("weight")
# dataloader.SetBackgroundWeightExpression("weight")

# Tell the dataloader how to use the training and testing events
#
# If no numbers of events are given, half of the events in the tree are used
# for training, and the other half for testing:
#
#    dataloader->PrepareTrainingAndTestTree( mycut, "SplitMode=random:!V" );
#
# To also specify the number of testing events, use:
#
#    dataloader->PrepareTrainingAndTestTree( mycut,
#         "NSigTrain=3000:NBkgTrain=3000:NSigTest=3000:NBkgTest=3000:SplitMode=Random:!V" );

# dataloader.SetSignalWeightExpression("xSection*(overallWeight/sumOfWeight)")
# dataloader.SetBackgroundWeightExpression("xSection*(overallWeight/sumOfWeight)")
dataloader.PrepareTrainingAndTestTree(TCut(''), "SplitMode=Random:NormMode=NumEvents:!V") #nTrain_Signal=5127:nTrain_Background=454:

# Generate model
### BDT
# factory.BookMethod( dataloader,  TMVA.Types.kBDT, "BDTG",
#   "!H:!V:NTrees=100:MinNodeSize=2.5%:MaxDepth=3:BoostType=AdaBoost:AdaBoostBeta=0.5:UseBaggedBoost:BaggedSampleFraction=0.5:SeparationType=GiniIndex:nCuts=20:VarTransform=D,P,N:")
### BDTG
# factory.BookMethod( dataloader,  TMVA.Types.kBDT, "BDTG",
#   "!H:!V:NTrees=1000:MinNodeSize=2.5%:BoostType=Grad:Shrinkage=0.10:nCuts=20:MaxDepth=2")
factory.BookMethod( dataloader,  TMVA.Types.kBDT, "BDTG",
  "!H:!V:VarTransform=D,P,N:NTrees=1000:MinNodeSize=2.5%:BoostType=Grad:Shrinkage=0.10:UseBaggedBoost:BaggedSampleFraction=0.5:nCuts=20:MaxDepth=2")
### ANN
# factory.BookMethod( dataloader, TMVA.Types.kMLP, "MLP_ANN", "!H:!V:VarTransform=D,P,N:NCycles=500:HiddenLayers=N+1,N+2,N+1,N:LearningRate=0.02:UseRegulator=False:EstimatorType=CE:TestRate=10:CalculateErrors=True")
factory.BookMethod( dataloader, TMVA.Types.kMLP, "MLP_ANN","H:!V:NeuronType=tanh:VarTransform=N:NCycles=600:HiddenLayers=N+5:TestRate=5" )
### DNN
# factory.BookMethod( dataloader, TMVA.Types.kDNN, "DNN", "!H:!V:VarTransform=D,P,N:Layout=RELU|10,RELU|20,RELU|15,LINEAR:TrainingStrategy=LearningRate=1e-1,BatchSize=1000|LearningRate=1e-2,BatchSize=1000|LearningRate=1e-3,BatchSize=1000,Regularization=L1")

# Run TMVA
factory.TrainAllMethods()
factory.TestAllMethods()
factory.EvaluateAllMethods()

output.Close()
# if (not gROOT.IsBatch()):
# TMVA.TMVAGui(outfileName)

# TMVA::TMVAGui("c2030.root")
